@@ -31,6 +31,18 @@ typedef struct
     zval *onSSLReady;
 #endif
 
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+    zend_fcall_info_cache cache_onConnect;
+    zend_fcall_info_cache cache_onReceive;
+    zend_fcall_info_cache cache_onClose;
+    zend_fcall_info_cache cache_onError;
+    zend_fcall_info_cache cache_onBufferFull;
+    zend_fcall_info_cache cache_onBufferEmpty;
+#ifdef SW_USE_OPENSSL
+    zend_fcall_info_cache cache_onSSLReady;
+#endif
+#endif
+
 #if PHP_MAJOR_VERSION >= 7
     zval _object;
     zval _onConnect;
@@ -102,32 +114,54 @@ static sw_inline void client_execute_callback(zval *zobject, enum php_swoole_cli
     client_callback *cb = swoole_get_property(zobject, client_property_callback);
     char *callback_name;
 
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+    zend_fcall_info_cache *fci_cache;
+#endif
+
     switch(type)
     {
     case SW_CLIENT_CB_onConnect:
         callback = cb->onConnect;
         callback_name = "onConnect";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onConnect;
+#endif
         break;
     case SW_CLIENT_CB_onError:
         callback = cb->onError;
         callback_name = "onError";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onError;
+#endif
         break;
     case SW_CLIENT_CB_onClose:
         callback = cb->onClose;
         callback_name = "onClose";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onClose;
+#endif
         break;
     case SW_CLIENT_CB_onBufferFull:
         callback = cb->onBufferFull;
         callback_name = "onBufferFull";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onBufferFull;
+#endif
         break;
     case SW_CLIENT_CB_onBufferEmpty:
         callback = cb->onBufferEmpty;
         callback_name = "onBufferEmpty";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onBufferEmpty;
+#endif
         break;
 #ifdef SW_USE_OPENSSL
     case SW_CLIENT_CB_onSSLReady:
         callback = cb->onSSLReady;
         callback_name = "onSSLReady";
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        fci_cache = &cb->cache_onSSLReady;
+#endif
         break;
 #endif
     default:
@@ -141,7 +175,11 @@ static sw_inline void client_execute_callback(zval *zobject, enum php_swoole_cli
     }
 
     args[0] = &zobject;
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+    if (sw_call_user_function_fast(callback, fci_cache, &retval, 1, args TSRMLS_CC) == FAILURE)
+#else
     if (sw_call_user_function_ex(EG(function_table), NULL, callback, &retval, 1, args, 0, NULL TSRMLS_CC) == FAILURE)
+#endif
     {
         swoole_php_fatal_error(E_WARNING, "%s handler error.", callback_name);
         return;
@@ -317,7 +355,11 @@ static void client_onReceive(swClient *cli, char *data, uint32_t length)
         goto free_zdata;
     }
 
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+    if (sw_call_user_function_fast(zcallback, &cb->cache_onReceive, &retval, 2, args TSRMLS_CC) == FAILURE)
+#else
     if (sw_call_user_function_ex(EG(function_table), NULL, zcallback, &retval, 2, args, 0, NULL TSRMLS_CC) == FAILURE)
+#endif
     {
         swoole_php_fatal_error(E_WARNING, "onReactorCallback handler error");
         goto free_zdata;
@@ -666,6 +708,7 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
             convert_to_string(v);
             cli->socks5_proxy->username = Z_STRVAL_P(v);
             cli->socks5_proxy->l_username = Z_STRLEN_P(v);
+            cli->socks5_proxy->method = 0x02;
         }
         if (php_swoole_array_get_value(vht, "socks5_password", v))
         {
@@ -1124,11 +1167,6 @@ static PHP_METHOD(swoole_client, connect)
             RETURN_FALSE;
         }
 
-        if (timeout > 0)
-        {
-            php_swoole_check_timer((int) (timeout * 1000));
-        }
-
         if (swSocket_is_stream(cli->type))
         {
             if (!cb->onConnect)
@@ -1708,7 +1746,17 @@ static PHP_METHOD(swoole_client, on)
         swoole_set_property(getThis(), client_property_callback, cb);
     }
 
-#ifdef PHP_SWOOLE_CHECK_CALLBACK
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+    char *func_name = NULL;
+    zend_fcall_info_cache func_cache;
+    if (!sw_zend_is_callable_ex(zcallback, NULL, 0, &func_name, NULL, &func_cache, NULL TSRMLS_CC))
+    {
+        swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+        efree(func_name);
+        return;
+    }
+    efree(func_name);
+#elif defined(PHP_SWOOLE_CHECK_CALLBACK)
     char *func_name = NULL;
     if (!sw_zend_is_callable(zcallback, 0, &func_name TSRMLS_CC))
     {
@@ -1724,36 +1772,54 @@ static PHP_METHOD(swoole_client, on)
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onConnect"), zcallback TSRMLS_CC);
         cb->onConnect = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onConnect"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onConnect, cb->_onConnect);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onConnect = func_cache;
+#endif
     }
     else if (strncasecmp("receive", cb_name, cb_name_len) == 0)
     {
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onReceive"), zcallback TSRMLS_CC);
         cb->onReceive = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onReceive"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onReceive, cb->_onReceive);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onReceive = func_cache;
+#endif
     }
     else if (strncasecmp("close", cb_name, cb_name_len) == 0)
     {
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onClose"), zcallback TSRMLS_CC);
         cb->onClose = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onClose"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onClose, cb->_onClose);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onClose = func_cache;
+#endif
     }
     else if (strncasecmp("error", cb_name, cb_name_len) == 0)
     {
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onError"), zcallback TSRMLS_CC);
         cb->onError = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onError"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onError, cb->_onError);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onError = func_cache;
+#endif
     }
     else if (strncasecmp("bufferFull", cb_name, cb_name_len) == 0)
     {
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onBufferFull"), zcallback TSRMLS_CC);
         cb->onBufferFull = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onBufferFull"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onBufferFull, cb->_onBufferFull);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onBufferFull = func_cache;
+#endif
     }
     else if (strncasecmp("bufferEmpty", cb_name, cb_name_len) == 0)
     {
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onBufferEmpty"), zcallback TSRMLS_CC);
         cb->onBufferEmpty = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onBufferEmpty"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onBufferEmpty, cb->_onBufferEmpty);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onBufferEmpty = func_cache;
+#endif
     }
     else
     {
@@ -1836,7 +1902,17 @@ static PHP_METHOD(swoole_client, enableSSL)
         {
             return;
         }
-#ifdef PHP_SWOOLE_CHECK_CALLBACK
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        char *func_name = NULL;
+        zend_fcall_info_cache func_cache;
+        if (!sw_zend_is_callable_ex(zcallback, NULL, 0, &func_name, NULL, &func_cache, NULL TSRMLS_CC))
+        {
+            swoole_php_fatal_error(E_ERROR, "Function '%s' is not callable", func_name);
+            efree(func_name);
+            return;
+        }
+        efree(func_name);
+#elif defined(PHP_SWOOLE_CHECK_CALLBACK)
         char *func_name = NULL;
         if (!sw_zend_is_callable(zcallback, 0, &func_name TSRMLS_CC))
         {
@@ -1856,6 +1932,9 @@ static PHP_METHOD(swoole_client, enableSSL)
         zend_update_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("onSSLReady"), zcallback TSRMLS_CC);
         cb->onSSLReady = sw_zend_read_property(swoole_client_class_entry_ptr,  getThis(), ZEND_STRL("onSSLReady"), 0 TSRMLS_CC);
         sw_copy_to_stack(cb->onSSLReady, cb->_onSSLReady);
+#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+        cb->cache_onSSLReady = func_cache;
+#endif
         cli->ssl_wait_handshake = 1;
         cli->socket->ssl_state = SW_SSL_STATE_WAIT_STREAM;
 
