@@ -621,7 +621,7 @@ static int http_client_onMessage(swConnection *conn, char *data, uint32_t length
     {
         sw_zval_ptr_dtor(&retval);
     }
-    sw_zval_ptr_dtor(&zframe);
+    sw_zval_free(zframe);
 
     return SW_OK;
 }
@@ -1213,9 +1213,7 @@ static PHP_METHOD(swoole_http_client, __construct)
     }
 
     zend_update_property_stringl(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("host"), host, host_len TSRMLS_CC);
-
-    zend_update_property_long(swoole_http_client_class_entry_ptr,
-    getThis(), ZEND_STRL("port"), port TSRMLS_CC);
+    zend_update_property_long(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("port"), port TSRMLS_CC);
 
     //init
     swoole_set_object(getThis(), NULL);
@@ -1949,12 +1947,6 @@ static PHP_METHOD(swoole_http_client, push)
         RETURN_FALSE;
     }
 
-    if (length == 0)
-    {
-        swoole_php_fatal_error(E_WARNING, "data is empty.");
-        RETURN_FALSE;
-    }
-
     http_client *http = swoole_get_object(getThis());
     if (!http->cli)
     {
